@@ -82,14 +82,6 @@ namespace Beagle.Util {
 
 		/////////////////////////////////////////////////////////////////////////////////////
 
-		private static Logger log;
-
-		public static Logger Log { 
-			get { return log; }
-		}
-
-		/////////////////////////////////////////////////////////////////////////////////////
-
 		[StructLayout (LayoutKind.Sequential)]
 		private struct inotify_event {
 			public int       wd;
@@ -122,8 +114,6 @@ namespace Beagle.Util {
 
 		static Inotify ()
 		{
-			log = Logger.Get ("Inotify");
-
 			if (Environment.GetEnvironmentVariable ("BEAGLE_DISABLE_INOTIFY") != null) {
 				Logger.Log.Debug ("BEAGLE_DISABLE_INOTIFY is set");
 				return;
@@ -139,8 +129,35 @@ namespace Beagle.Util {
 				return;
 			}
 
-			if (inotify_fd == -1)
-				Logger.Log.Warn ("Could not initialize inotify");
+			if (inotify_fd < 0) {
+				Mono.Unix.Native.Errno errno = Mono.Unix.Native.NativeConvert.ToErrno (-inotify_fd);
+
+				string error_message;
+
+				if (errno == Mono.Unix.Native.Errno.ENOSYS)
+					error_message = "Inotify not supported!  You need a 2.6.13 kernel or later with CONFIG_INOTIFY enabled.";
+				else
+					error_message = Mono.Unix.UnixMarshal.GetErrorDescription (errno);
+
+				Logger.Log.Warn ("Could not initialize inotify: {0}", error_message);
+			} else {
+				try {
+					FileStream fs = new FileStream ("/proc/sys/fs/inotify/max_user_watches", FileMode.Open, FileAccess.Read);
+					StreamReader r = new StreamReader (fs);
+
+					string line = r.ReadLine ();
+					r.Close ();
+
+					int watches = -1;
+					try {
+						watches = Int32.Parse (line);
+					} catch (FormatException) { }
+					
+					if (watches > -1 && watches < 32768) {
+						Log.Warn ("Inotify watches may be too low ({0}) for some users!  Increase it to at least 65535 by setting fs.inotify.max_user_watches in /etc/sysctl.conf", watches);
+					}
+				} catch { }
+			}
 		}
 
 		public static bool Enabled {
@@ -414,7 +431,8 @@ namespace Beagle.Util {
 
 			int retval = inotify_glue_ignore (inotify_fd, watched.Wd);
 			if (retval < 0) {
-				string msg = String.Format ("Attempt to ignore {0} failed!", watched.Path);
+				Mono.Unix.Native.Errno errno = Mono.Unix.Native.NativeConvert.ToErrno (-retval);
+				string msg = String.Format ("Attempt to ignore {0} failed: {1}", watched.Path, Mono.Unix.UnixMarshal.GetErrorDescription (errno));
 				throw new IOException (msg);
 			}
 
@@ -422,6 +440,10 @@ namespace Beagle.Util {
 			return;
 		}
 
+		// We don't want to keep printing the inotify watch limit
+		// warning over and over again.
+		static bool watch_limit_error_displayed = false;
+
 		// Ensure our watch exists, meets all the subscribers requirements,
 		// and isn't matching any other events that we don't care about.
 		private static void CreateOrModifyWatch (WatchInfo watched)
@@ -433,6 +455,8 @@ namespace Beagle.Util {
 			if (watched.Wd >= 0 && watched.Mask == new_mask)
 				return;
 
+			//Log.Debug ("{0} inotify watch on {1}", watched.Wd >= 0 ? "Recreating" : "Creating", watched.Path);
+
 			// We rely on the behaviour that watching the same inode twice won't result
 			// in the wd value changing.
 			// (no need to worry about watched_by_wd being polluted with stale watches)
@@ -440,9 +464,17 @@ namespace Beagle.Util {
 			int wd = -1;
 			wd = inotify_glue_watch (inotify_fd, watched.Path, new_mask);
 			if (wd < 0) {
-				string msg = String.Format ("Attempt to watch {0} failed!", watched.Path);
+				Mono.Unix.Native.Errno errno = Mono.Unix.Native.NativeConvert.ToErrno (-wd);
+
+				if (! watch_limit_error_displayed && errno == Mono.Unix.Native.Errno.ENOSPC) {
+					Log.Error ("Maximum inotify watch limit hit adding watch to {0}.  Try adjusting /proc/sys/fs/inotify/max_user_watches", watched.Path);
+					watch_limit_error_displayed = true;
+				}
+
+				string msg = String.Format ("Attempt to watch {0} failed: {1}", watched.Path, Mono.Unix.UnixMarshal.GetErrorDescription (errno));
 				throw new IOException (msg);
 			}
+
 			if (watched.Wd >= 0 && watched.Wd != wd) {
 				string msg = String.Format ("Watch handle changed unexpectedly!", watched.Path);	
 				throw new IOException (msg);
