@@ -473,7 +473,7 @@ namespace Beagle.Daemon {
 
 			// Check if global configuration files are installed
 			if (! Conf.CheckGlobalConfig ()) {
-				Console.WriteLine ("Global configuration files not found in '{0}'", PathFinder.GlobalConfigDir);
+				Console.WriteLine ("Global configuration files not found in '{0}'", PathFinder.ConfigDataDir);
 				Environment.Exit (-1);
 			}
 
@@ -500,7 +500,12 @@ namespace Beagle.Daemon {
 			QueryDriver.Init ();
 			Server.Init ();
 
+#if MONO_1_9
+			Shutdown.SetupSignalHandlers (new Shutdown.SignalHandler (HandleSignal));
+#else
 			SetupSignalHandlers ();
+#endif
+
 			Shutdown.ShutdownEvent += OnShutdown;
 
 			main_loop = new MainLoop ();
@@ -566,6 +571,13 @@ namespace Beagle.Daemon {
 
 			// Ignore SIGPIPE
 			Mono.Unix.Native.Stdlib.signal (Mono.Unix.Native.Signum.SIGPIPE, Mono.Unix.Native.Stdlib.SIG_IGN);
+
+			// Work around a mono feature/bug
+			// https://bugzilla.novell.com/show_bug.cgi?id=381928
+			// When beagle crashes, mono will try to print a stack trace and then call abort()
+			// The abort somehow calls back into beagle and causes a deadlock
+			if (Environment.GetEnvironmentVariable ("BEAGLE_MONO_DEBUG_FLAG_IS_SET") == null)
+				Mono.Unix.Native.Stdlib.signal (Mono.Unix.Native.Signum.SIGABRT, Mono.Unix.Native.Stdlib.SIG_DFL);
 		}
 
 		// Mono signal handler allows setting of global variables;
@@ -578,7 +590,8 @@ namespace Beagle.Daemon {
 				return;
 
 			// Set shutdown flag to true so that other threads can stop initializing
-			if ((Mono.Unix.Native.Signum) signal != Mono.Unix.Native.Signum.SIGUSR1)
+			if ((Mono.Unix.Native.Signum) signal != Mono.Unix.Native.Signum.SIGUSR1 &&
+			    (Mono.Unix.Native.Signum) signal != Mono.Unix.Native.Signum.SIGUSR2)
 				Shutdown.ShutdownRequested = true;
 
 			// Do all signal handling work in the main loop and not in the signal handler.
@@ -589,17 +602,22 @@ namespace Beagle.Daemon {
 		{
 			Logger.Log.Debug ("Handling signal {0} ({1})", signal, (Mono.Unix.Native.Signum) signal);
 
+			// Pass the signals to the helper too.
+			GLib.Idle.Add (new GLib.IdleHandler (delegate ()
+							    {
+								    RemoteIndexer.SignalRemoteIndexer ((Mono.Unix.Native.Signum) signal);
+								    return false; 
+							    }));
+
 			// If we get SIGUSR1, turn the debugging level up.
 			if ((Mono.Unix.Native.Signum) signal == Mono.Unix.Native.Signum.SIGUSR1) {
 				LogLevel old_level = Log.Level;
 				Log.Level = LogLevel.Debug;
 				Log.Debug ("Moving from log level {0} to Debug", old_level);
-			}
-
-			// Send informational signals to the helper too.
-			if ((Mono.Unix.Native.Signum) signal == Mono.Unix.Native.Signum.SIGUSR1 ||
-			    (Mono.Unix.Native.Signum) signal == Mono.Unix.Native.Signum.SIGUSR2) {
-				GLib.Idle.Add (new GLib.IdleHandler (delegate () { RemoteIndexer.SignalRemoteIndexer ((Mono.Unix.Native.Signum) signal); return false; }));
+				return;
+			} else if ((Mono.Unix.Native.Signum) signal == Mono.Unix.Native.Signum.SIGUSR2) {
+				// Debugging hook for beagled
+				QueryDriver.DebugHook ();
 				return;
 			}
 
@@ -627,7 +645,6 @@ namespace Beagle.Daemon {
 
 		/////////////////////////////////////////////////////////////////////////////
 
-
 		private static ArrayList exercise_files = new ArrayList ();
 
 		private static void ExerciseTheDogHarder ()
