@@ -38,7 +38,6 @@ using Beagle.Util;
 namespace Beagle.Daemon.FileSystemQueryable {
 
 	[QueryableFlavor (Name="Files", Domain=QueryDomain.Local | QueryDomain.Neighborhood, RequireInotify=false)]
-	[PropertyKeywordMapping (Keyword="ext", PropertyName="beagle:FilenameExtension", IsKeyword=true, Description="File extension, e.g. ext:jpeg. Use ext: to search in files with no extension.")]
 	public class FileSystemQueryable : LuceneQueryable {
 
 		static internal bool Debug = false;
@@ -220,7 +219,7 @@ namespace Beagle.Daemon.FileSystemQueryable {
 		public Indexable FileToIndexable (string         path,
 						  Guid           id,
 						  DirectoryModel parent,
-						  bool           crawl_mode)
+						  bool           crawling)
 		{
 			Indexable indexable;
 
@@ -245,7 +244,7 @@ namespace Beagle.Daemon.FileSystemQueryable {
 
 			indexable.ContentUri = UriFu.PathToFileUri (path);
 			indexable.DisplayUri = UriFu.PathToFileUri (path);
-			indexable.Crawled = crawl_mode;
+			indexable.FlushBufferCache = crawling;
 			indexable.Filtering = Beagle.IndexableFiltering.Always;
 
 			FileInfo fi = new FileInfo (path);
@@ -258,7 +257,7 @@ namespace Beagle.Daemon.FileSystemQueryable {
 			indexable.LocalState ["Path"] = path;
 
 			MergeExternalPendingIndexable (indexable);
-			Indexable xmp_indexable = xmp_handler.MergeXmpData (ref indexable, path, id, parent, crawl_mode);
+			Indexable xmp_indexable = xmp_handler.MergeXmpData (ref indexable, path, id, parent, crawling);
 
 			// In full generality, the xmp_handler can request an entirely new indexable to be scheduled.
 			// So, we should do something with the returned xmp_indexable if it is not null.
@@ -465,6 +464,8 @@ namespace Beagle.Daemon.FileSystemQueryable {
 
 			lock (dir_models_by_path)
 				dir_models_by_path.Remove (expired_path);
+
+			FileAttributesStore.Drop (expired_path);
 		}
 
 		public void AddDirectory (DirectoryModel parent, string name)
@@ -1028,6 +1029,14 @@ namespace Beagle.Daemon.FileSystemQueryable {
 						self_task = null;
 				}
 			}
+
+			internal void DebugHook ()
+			{
+				lock (queue_lock)
+					Log.Debug ("FSQ:FileSystemEventsGenerator Debughook: {0} more events to process, generator is {1}active",
+						event_queue.Count,
+						(self_task != null ? String.Empty : "in"));
+			}
 		}
 
 		//////////////////////////////////////////////////////////////////////////
@@ -1844,7 +1853,7 @@ namespace Beagle.Daemon.FileSystemQueryable {
 			return parent_dirs;
 		}
 
-		override public ISnippetReader GetSnippet (string [] query_terms, Hit hit, bool full_text)
+		override public ISnippetReader GetSnippet (string [] query_terms, Hit hit, bool full_text, int ctx_length, int snp_length)
 		{
 			// Uri remapping from a hit is easy: the internal uri
 			// is stored in a property.
@@ -1854,12 +1863,12 @@ namespace Beagle.Daemon.FileSystemQueryable {
 			TextReader reader = TextCache.UserCache.GetReader (uri, ref self_cache);
 
 			if (self_cache)
-				return SnippetFu.GetSnippetFromFile (query_terms, hit.Uri.LocalPath, full_text);
+				return SnippetFu.GetSnippetFromFile (query_terms, hit.Uri.LocalPath, full_text, ctx_length, snp_length);
 
 			if (reader == null)
 				return null;
 
-			return SnippetFu.GetSnippet (query_terms, reader, full_text);
+			return SnippetFu.GetSnippet (query_terms, reader, full_text, ctx_length, snp_length);
 		}
 
 		override public void Start ()
@@ -2015,6 +2024,45 @@ namespace Beagle.Daemon.FileSystemQueryable {
 
 			return internal_uri;
 		}
+
+		//////////////////////////
+
+		internal override void DebugHook ()
+		{
+			Log.Debug ("FSQ DebugHook: {0} roots, {1} roots by path, {2} dir_models_by_id, {3} name_info_by_id, {4} dir_models_by_path",
+				(roots != null ? roots.Count : -1),
+				(roots_by_path != null ? roots_by_path.Count : -1),
+				(dir_models_by_id != null ? dir_models_by_id.Count : -1),
+				(name_info_by_id != null ? name_info_by_id.Count : -1),
+				(dir_models_by_path != null ? dir_models_by_path.Count : -1));
+
+			Log.Debug ("FSQ Debughook: {0} external_pending_indexables",
+				(external_pending_indexables != null ? external_pending_indexables.Count : -1));
+
+			lock (big_lock) {
+				foreach (DirectoryModel root in roots) {
+					int count = 1;
+					ArrayList queue = new ArrayList ();
+					queue.Add (root);
+					DirectoryModel child;
+					while (queue.Count > 0) {
+						child = (DirectoryModel) queue [0];
+						queue.RemoveAt (0);
+
+						if (child.ChildCount == 0)
+							continue;
+						count += child.ChildCount;
+						queue.AddRange (child.Children);
+					}
+					Console.WriteLine ("FSQ Debughook: {0} has {1} subdirs", root.FullName, count);
+				}
+			}
+
+			tree_crawl_task.DebugHook ();
+			file_crawl_task.DebugHook ();
+			fs_event_generator.DebugHook ();
+			uid_manager.DebugHook ();
+		}
 	}
 }
 	
